; Syntax: NASM, tab width = 8 spaces (or higher)
; - 8bitbubsy, 2023

	CPU 8086
	ORG 100h

VIDEO_SEG	EQU 0A000h
SCREEN_W 	EQU 320
SCREEN_H 	EQU 200
R_COL_OFFSET	EQU (2+(63*0))
G_COL_OFFSET	EQU (2+(63*1))
B_COL_OFFSET	EQU (2+(63*2))
C_COL_OFFSET	EQU (2+(63*3))
KEY_ESC		EQU 27

Main:
	mov ax,VIDEO_SEG
	mov es,ax
	; ------------------- 
	call DetectVGAWithColorMonitor
	jnc .L0
	lea dx,ErrorText	; print error msg
	mov ah,9
	int 21h
	jmp ExitToDOS
.L0:	; ------------------- get current video mode
	mov ax,0F00h
	int 10h
	mov [OldVideoMode],al
	; ------------------- enter 320x200 @ 70Hz (chunky mode)
	mov ax,0013h
	int 10h
	; ------------------- set black color palette
	mov ax,1
	xor bx,bx
	xor cx,cx
	xor dx,dx
	call SetPaletteColor
	; ------------------- set other palette entries
	call SetColorGradientsPalette
	call SetGrayscalePalette
	; -------------------
	call DrawGradientBars
	; -------------------
.loop:	mov ah,8 ; get key
	int 21h
	cmp al,KEY_ESC
	je .done
	call SwitchTestMode
	jmp .loop
.done:	; ------------------- set back old video mode
	xor ax,ax
	mov al,[OldVideoMode]
	int 10h
ExitToDOS:
	mov ah,4Ch
	int 21h

DetectVGAWithColorMonitor:
	mov ax,1A00h
	int 10h
	cmp al,26
	jne .error
	cmp bl,8
	je .ok
	cmp bl,10
	je .ok
	cmp bl,12
	je .ok
.error:	stc
	ret
.ok:	clc
	ret

WaitForVblank:
	mov dx,3DAh
.L0:	in al,dx
	test al,8
	jz .L0
.L1:	in al,dx
	test al,8
	jnz .L1
	ret

	; al = index, bl = red, cl = green, dl = blue
SetPaletteColor:
	push ax
	push dx
	mov bp,sp
	mov al,0FFh
	mov dx,03C6h
	out dx,al
	add dx,2
	mov ax,[bp+2]
	out dx,al
	inc dx
	mov al,bl
	out dx,al
	mov al,cl
	out dx,al
	mov ax,[bp+0]
	out dx,al
	pop dx
	pop ax
	ret

%macro SET_PAL 4
	mov al,%1
	mov bl,%2
	mov cl,%3
	mov dl,%4
	call SetPaletteColor
%endmacro

SwitchTestMode:
	mov al,[TestMode]
	inc al
	cmp al,6
	jne .L0
	xor al,al
.L0:	mov [TestMode],al
	; -------------------
	cmp al,0
	jne .L1
	call DrawGradientBars
	ret
.L1:	cmp al,1
	jne .L2
	call DrawColorBars
	ret
.L2:	cmp al,2
	jne .L3
	; ------------------- red screen
	xor di,di ; clear screen first
	xor ax,ax
	mov cx,(SCREEN_W*SCREEN_H)/2
	rep stosw
	; -------------------
	call WaitForVblank
	SET_PAL 0,63,0,0
	ret
.L3:	cmp al,3
	jne .L4
	; ------------------- green screen
	call WaitForVblank
	SET_PAL 0,0,63,0
	ret
.L4:	cmp al,4
	jne .L5
	; ------------------- blue screen
	call WaitForVblank
	SET_PAL 0,0,0,63
.L5:	cmp al,5
	jne .L6
	; ------------------- white screen
	call WaitForVblank
	SET_PAL 0,63,63,63
.L6:	ret

SetGrayscalePalette:
	mov si,62
.loop:	lea ax,[si+C_COL_OFFSET]
	mov bx,si
	mov cx,si
	mov dx,si
	call SetPaletteColor
	dec si
	jns .loop
	ret

SetColorGradientsPalette:
	mov si,62
.loop:	; ------------------- red gradient
	lea ax,[si+R_COL_OFFSET]
	mov bx,si
	xor cx,cx
	xor dx,dx
	call SetPaletteColor
	; ------------------- green gradient
	lea ax,[si+G_COL_OFFSET]
	xor bx,bx
	mov cx,si
	xor dx,dx
	call SetPaletteColor
	; ------------------- blue gradient
	lea ax,[si+B_COL_OFFSET]
	xor bx,bx
	xor cx,cx
	mov dx,si
	call SetPaletteColor
	; -------------------
	dec si
	jns .loop
	ret

DrawGradientBar:
	mov si,(SCREEN_W/5)-1	; 5 (320/64) pixels per color
.yloop:	mov cx,si
	mov ax,0101h ; use special palette entry for black entry (kludge)
	stosw
	stosw
	stosb
	mov ax,bx
.xloop:	mov ah,al
	stosw
	stosw
	stosb	
	inc al
	dec cx
	jne .xloop
	dec dx
	jne .yloop
	ret

DrawGradientBars:
	call WaitForVblank
	SET_PAL 0,63,63,63 ; white background
	call SetColorGradientsPalette
	xor di,di
	; ------------------- red gradient bar
	mov bx,R_COL_OFFSET
	mov dx,50 ; height
	call DrawGradientBar
	; ------------------- green gradient bar
	mov bx,G_COL_OFFSET
	mov dx,50
	call DrawGradientBar
	; ------------------- blue gradient bar
	mov bx,B_COL_OFFSET
	mov dx,50
	call DrawGradientBar
	; ------------------- grayscale gradient bar
	mov bx,C_COL_OFFSET
	mov dx,50
	call DrawGradientBar
	ret

DrawColorBars:
	call WaitForVblank
	SET_PAL 0,0,0,0 ; black background
	SET_PAL	R_COL_OFFSET+0, 63,  63,  63 ; white
	SET_PAL R_COL_OFFSET+1, 63,  63,   0 ; yellow
	SET_PAL R_COL_OFFSET+2,  0,  63,  63 ; cyan
	SET_PAL R_COL_OFFSET+3,  0,  63,   0 ; green
	SET_PAL R_COL_OFFSET+4, 63,   0,  63 ; magenta
	SET_PAL R_COL_OFFSET+5, 63,   0,   0 ; red
	SET_PAL R_COL_OFFSET+6,  0,   0,  63 ; blue
	SET_PAL R_COL_OFFSET+7,  4,   4,   4 ; dark gray
	xor di,di
	; -------------------
	mov si,(SCREEN_W/8)/2
	mov dx,SCREEN_H-50
.yloop:	mov ax,((R_COL_OFFSET+0)*256)+(R_COL_OFFSET+0)
	mov bx,((R_COL_OFFSET+7)*256)+(R_COL_OFFSET+7)
.xloop:	mov cx,si
	rep stosw
	add ax,0101h
	cmp ax,bx
	jbe .xloop
	dec dx
	jne .yloop
	ret

; -----------------------------------------------------------------------------
; -----------------------------------------------------------------------------

OldVideoMode:	db 0
TestMode:	db 0

ErrorText:
	db "ERROR: This program requires a VGA graphics adapter and a color "
	db "monitor.",13,10,"$"
